@@ -592,7 +592,8 @@ static int init_drive(struct harddisk_info *info, struct controller *ctrl, int s
 			info->sectors_per_track *
 			info->heads *
 			info->cylinders;
-		printf(__FUNCTION__ " sectors_per_track=[%d], heads=[%d], cylinders=[%d]\n",
+		printf( "%s sectors_per_track=[%d], heads=[%d], cylinders=[%d]\n",
+			__FUNCTION__,
 			info->sectors_per_track,
 			info->heads,
 			info->cylinders);
@@ -657,7 +658,7 @@ static int init_controller(struct controller *ctrl, int basedrive, unsigned char
 	 * 
 	 */
 #if !BSY_SET_DURING_SPINUP
-	if (await_ide(timeout, ctrl, currticks() + IDE_TIMEOUT) < 0) {
+	if (await_ide(timeout, ctrl, IDE_TIMEOUT) < 0) {
 		return -1;
 	}
 #endif
@@ -756,6 +757,7 @@ static int ide_pci_probe(struct dev *dev, struct pci_device *pci)
 	dev->index = 0;
 	return 0;
 }
+#define PCI_DEVICE_ID_INTEL_82801CA_11	0x248b
 static struct pci_id ide_controllers[] = {
 { PCI_VENDOR_ID_INTEL,       PCI_DEVICE_ID_INTEL_82801CA_11,    "PIIX4" },
 #if 0  /* Currently I don't need any entries in this table so ignore it */
