@@ -1,6 +1,25 @@
 #ifdef ALLMULTI
 #error multicast support is not yet implemented
 #endif
+
+/**
+   Per an email message from Russ Nelson <nelson@crynwr.com> on
+   18 March 2008 this file is now licensed under GPL Version 2.
+
+   From: Russ Nelson <nelson@crynwr.com>
+   Date: Tue, 18 Mar 2008 12:42:00 -0400
+   Subject: Re: [Etherboot-developers] cs89x0 driver in etherboot
+   -- quote from email 
+   As copyright holder, if I say it doesn't conflict with the GPL,
+   then it doesn't conflict with the GPL.
+
+   However, there's no point in causing people's brains to overheat,
+   so yes, I grant permission for the code to be relicensed under the
+   GPLv2.  Please make sure that this change in licensing makes its
+   way upstream.  -russ 
+   -- quote from email
+**/
+
 /* cs89x0.c: A Crystal Semiconductor CS89[02]0 driver for etherboot. */
 /*
   Permission is granted to distribute the enclosed cs89x0.[ch] driver
@@ -77,7 +96,7 @@ static unsigned short	eth_nic_base;
 static unsigned long	eth_nic_base;
 #endif
 static unsigned long    eth_mem_start;
-static unsigned short   eth_irq;
+static unsigned short   eth_irqno;
 static unsigned short   eth_cs_type;	/* one of: CS8900, CS8920, CS8920M  */
 static unsigned short   eth_auto_neg_cnf;
 static unsigned short   eth_adapter_cnf;
@@ -321,9 +340,9 @@ static void cs89x0_reset(struct nic *nic)
 	if (eth_cs_type != CS8900) {
 		/* Hardware problem requires PNP registers to be reconfigured
 		   after a reset */
-		if (eth_irq != 0xFFFF) {
+		if (eth_irqno != 0xFFFF) {
 			outw(PP_CS8920_ISAINT, eth_nic_base + ADD_PORT);
-			outb(eth_irq, eth_nic_base + DATA_PORT);
+			outb(eth_irqno, eth_nic_base + DATA_PORT);
 			outb(0, eth_nic_base + DATA_PORT + 1); }
 
 		if (eth_mem_start) {
@@ -419,7 +438,7 @@ retry:
 ETH_POLL - Wait for a frame
 ***************************************************************************/
 
-static int cs89x0_poll(struct nic *nic)
+static int cs89x0_poll(struct nic *nic, int retrieve)
 {
 	int status;
 
@@ -428,6 +447,8 @@ static int cs89x0_poll(struct nic *nic)
 	if ((status & RX_OK) == 0)
 		return(0);
 
+	if ( ! retrieve ) return 1;
+
 	status = inw(eth_nic_base + RX_FRAME_PORT);
 	nic->packetlen = inw(eth_nic_base + RX_FRAME_PORT);
 	insw(eth_nic_base + RX_FRAME_PORT, nic->packet, nic->packetlen >> 1);
@@ -442,6 +463,18 @@ static void cs89x0_disable(struct dev *dev)
 	cs89x0_reset(nic);
 }
 
+static void cs89x0_irq(struct nic *nic __unused, irq_action_t action __unused)
+{
+  switch ( action ) {
+  case DISABLE :
+    break;
+  case ENABLE :
+    break;
+  case FORCE :
+    break;
+  }
+}
+
 /**************************************************************************
 ETH_PROBE - Look for an adapter
 ***************************************************************************/
@@ -533,7 +566,7 @@ static int cs89x0_probe(struct dev *dev, unsigned short *probe_addrs __unused)
 		    /* Check if the ISA IRQ has been set  */
 		    (i = readreg(PP_CS8920_ISAINT) & 0xff,
 		     (i != 0 && i < CS8920_NO_INTS)))
-			eth_irq = i;
+			eth_irqno = i;
 		else {
 			i = isa_cnf & INT_NO_MASK;
 			if (eth_cs_type == CS8900) {
@@ -547,7 +580,7 @@ static int cs89x0_probe(struct dev *dev, unsigned short *probe_addrs __unused)
 				   mapping of the table below. */
 				if (i < 4) i = "\012\013\014\005"[i];
 				else printf("\ncs: BUG: isa_config is %d\n", i); }
-			eth_irq = i; }
+			eth_irqno = i; }
 
 		/* Retrieve and print the ethernet address. */
 		for (i=0; i<ETH_ALEN; i++) {
@@ -558,12 +591,12 @@ static int cs89x0_probe(struct dev *dev, unsigned short *probe_addrs __unused)
 #ifdef EMBEDDED
 		/* Retrieve and print the ethernet address. */
 		{
-			unsigned char MAC_HW_ADDR[6]={MAC_HW_ADDR_DRV};
-			memcpy(nic->node_addr, MAC_HW_ADDR, 6);
+		unsigned char MAC_HW_ADDR[6]={MAC_HW_ADDR_DRV};
+		memcpy(nic->node_addr, MAC_HW_ADDR, 6);
 		}
 		printf("\n%!\n", nic->node_addr);
 
-		eth_adapter_cnf = A_CNF_10B_T | A_CNF_MEDIA_10B_T;
+	    eth_adapter_cnf = A_CNF_10B_T | A_CNF_MEDIA_10B_T;
 		eth_auto_neg_cnf = EE_AUTO_NEG_ENABLE | IMM_BIT;
 #endif
 #ifndef EMBEDDED 
@@ -677,9 +710,13 @@ static int cs89x0_probe(struct dev *dev, unsigned short *probe_addrs __unused)
 	if (ioaddr == 0)
 		return (0);
 
+	nic->irqno    = 0;
+	nic->ioaddr   = ioaddr;
+
 	dev->disable  = cs89x0_disable;
 	nic->poll     = cs89x0_poll;
 	nic->transmit = cs89x0_transmit;
+	nic->irq      = cs89x0_irq;
 
 	/* Based on PnP ISA map */
 	dev->devid.vendor_id = htons(ISAPNP_VENDOR('C','S','C'));
