@@ -82,6 +82,10 @@ unsigned long BASE;
 #define PCI_DEVICE_ID_NVIDIA_NVENET_9           0x0057
 #define PCI_DEVICE_ID_NVIDIA_NVENET_10          0x0037
 #define PCI_DEVICE_ID_NVIDIA_NVENET_11          0x0038
+#define PCI_DEVICE_ID_NVIDIA_NVENET_12          0x0268
+#define PCI_DEVICE_ID_NVIDIA_NVENET_13          0x0269
+#define PCI_DEVICE_ID_NVIDIA_NVENET_14          0x0372
+#define PCI_DEVICE_ID_NVIDIA_NVENET_15          0x0373
 
 
 /*
@@ -123,7 +127,8 @@ enum {
 	NvRegMisc1 = 0x080,
 #define NVREG_MISC1_HD		0x02
 #define NVREG_MISC1_FORCE	0x3b0f3c
-
+	NvRegMacReset = 0x3c,
+#define NVREG_MAC_RESET_ASSERT	0x0F3
 	NvRegTransmitterControl = 0x084,
 #define NVREG_XMITCTL_START	0x01
 	NvRegTransmitterStatus = 0x088,
@@ -234,6 +239,10 @@ enum {
 #define NVREG_POWERSTATE_D1		0x0001
 #define NVREG_POWERSTATE_D2		0x0002
 #define NVREG_POWERSTATE_D3		0x0003
+
+	NvRegPowerState2 = 0x600,
+#define NVREG_POWERSTATE2_POWERUP_MASK		0x0F11
+#define NVREG_POWERSTATE2_POWERUP_REV_A3	0x0001
 };
 
 #define FLAG_MASK_V1 0xffff0000
@@ -293,8 +302,8 @@ enum {
 #define NV_RX2_AVAIL		(1<<31)
 
 /* Miscelaneous hardware related defines: */
-#define NV_PCI_REGSZ		0x270
-
+#define NV_PCI_REGSZ_VER1	0x270
+#define NV_PCI_REGSZ_VER2	0x604
 /* various timeout delays: all in usec */
 #define NV_TXRX_RESET_DELAY	4
 #define NV_TXSTOP_DELAY1	10
@@ -310,7 +319,7 @@ enum {
 #define NV_MIIBUSY_DELAY	50
 #define NV_MIIPHY_DELAY	10
 #define NV_MIIPHY_DELAYMAX	10000
-
+#define NV_MAC_RESET_DELAY	64
 #define NV_WAKEUPPATTERNS	5
 #define NV_WAKEUPMASKENTRIES	4
 
@@ -439,6 +448,26 @@ struct forcedeth_private {
 
 static struct forcedeth_private *np;
 
+static void drop_rx(void);
+static void nv_udelay(unsigned long delay)
+{
+	if (!np->in_shutdown) {
+		udelay(delay);
+	} else while(delay) {
+		/* Don't allow an rx_ring overflow to happen
+		 * while shutting down the NIC it will
+		 * kill the receive function.
+		 */
+		unsigned long sleep;
+		drop_rx();
+		sleep = 3;
+		if (sleep > delay)
+			sleep = delay;
+		udelay(sleep);
+		delay -= sleep;
+	}
+}
+
 static inline void pci_push(u8 * base)
 {
 	/* force out pending posted writes */
@@ -458,7 +487,7 @@ static int reg_delay(int offset, u32 mask,
 
 	pci_push(base);
 	do {
-		udelay(delay);
+		nv_udelay(delay);
 		delaymax -= delay;
 		if (delaymax < 0) {
 			if (msg)
@@ -507,7 +536,7 @@ static int mii_rw(struct nic *nic __unused, int addr, int miireg,
 	reg = readl(base + NvRegMIIControl);
 	if (reg & NVREG_MIICTL_INUSE) {
 		writel(NVREG_MIICTL_INUSE, base + NvRegMIIControl);
-		udelay(NV_MIIBUSY_DELAY);
+		nv_udelay(NV_MIIBUSY_DELAY);
 	}
 
 	reg =
@@ -676,7 +705,7 @@ static void stop_rx(void)
 		  NV_RXSTOP_DELAY1, NV_RXSTOP_DELAY1MAX,
 		  "stop_rx: ReceiverStatus remained busy");
 
-	udelay(NV_RXSTOP_DELAY2);
+	nv_udelay(NV_RXSTOP_DELAY2);
 	writel(0, base + NvRegLinkSpeed);
 }
 
@@ -699,10 +728,26 @@ static void stop_tx(void)
 		  NV_TXSTOP_DELAY1, NV_TXSTOP_DELAY1MAX,
 		  "stop_tx: TransmitterStatus remained busy");
 
-	udelay(NV_TXSTOP_DELAY2);
+	nv_udelay(NV_TXSTOP_DELAY2);
 	writel(0, base + NvRegUnknownTransmitterReg);
 }
 
+static void mac_reset(struct nic *dev)
+{
+	u8 *base = (u8 *) BASE;
+
+	dprintf("mac_reset\n");
+	writel(NVREG_TXRXCTL_BIT2 | NVREG_TXRXCTL_RESET | np->desc_ver, base + NvRegTxRxControl);
+	pci_push(base);
+	writel(NVREG_MAC_RESET_ASSERT, base + NvRegMacReset);
+	pci_push(base);
+	udelay(NV_MAC_RESET_DELAY);
+	writel(0, base + NvRegMacReset);
+	pci_push(base);
+	udelay(NV_MAC_RESET_DELAY);
+	writel(NVREG_TXRXCTL_BIT2 | np->desc_ver, base + NvRegTxRxControl);
+	pci_push(base);
+}
 
 static void txrx_reset(struct nic *nic __unused)
 {
@@ -713,7 +758,7 @@ static void txrx_reset(struct nic *nic __unused)
 	       base + NvRegTxRxControl);
 
 	pci_push(base);
-	udelay(NV_TXRX_RESET_DELAY);
+	nv_udelay(NV_TXRX_RESET_DELAY);
 	writel(NVREG_TXRXCTL_BIT2 | np->desc_ver, base + NvRegTxRxControl);
 	pci_push(base);
 }
@@ -723,26 +768,47 @@ static void txrx_reset(struct nic *nic __unused)
  * Return 1 if the allocations for the skbs failed and the
  * rx engine is without Available descriptors
  */
-static int alloc_rx(struct nic *nic __unused)
+static void alloc_rx(struct nic *nic __unused)
 {
 	unsigned int refill_rx = np->refill_rx;
-	int i;
-	//while (np->cur_rx != refill_rx) {
-	for (i = 0; i < RX_RING; i++) {
-		//int nr = refill_rx % RX_RING;
-		rx_ring[i].PacketBuffer =
-		    virt_to_le32desc(&rxb[i * RX_NIC_BUFSIZE]);
+	while (np->cur_rx != refill_rx) {
+		int nr = refill_rx % RX_RING;
+		rx_ring[nr].PacketBuffer =
+		    virt_to_le32desc(&rxb[nr * RX_NIC_BUFSIZE]);
 		wmb();
-		rx_ring[i].FlagLen =
+		rx_ring[nr].FlagLen =
 		    cpu_to_le32(RX_NIC_BUFSIZE | NV_RX_AVAIL);
 		/*      printf("alloc_rx: Packet  %d marked as Available\n",
 		   refill_rx); */
 		refill_rx++;
 	}
 	np->refill_rx = refill_rx;
-	if (np->cur_rx - refill_rx == RX_RING)
-		return 1;
-	return 0;
+}
+
+static void drop_rx(void)
+{
+	u32 events;
+	u8 *base = (u8 *)BASE;
+
+	events = readl(base + NvRegIrqStatus);
+	if (events)
+		writel(events, base + NvRegIrqStatus);
+	if (!(events & (NVREG_IRQ_RX_ERROR|NVREG_IRQ_RX|NVREG_IRQ_RX_NOBUF)))
+		return;
+	for (;;) {
+		int i, len;
+		u32 Flags;
+		i = np->cur_rx % RX_RING;
+
+		Flags = le32_to_cpu(rx_ring[i].FlagLen);
+		len = nv_descr_getlength(&rx_ring[i], np->desc_ver);
+
+		if (Flags & NV_RX_AVAIL)
+			break;  /* still owned by hardware, */
+		wmb();
+		np->cur_rx++;
+		alloc_rx(NULL);
+	}
 }
 
 static int update_linkspeed(struct nic *nic)
@@ -894,7 +960,7 @@ static void nv_linkchange(struct nic *nic)
 }
 
 
-static int init_ring(struct nic *nic)
+static void init_ring(struct nic *nic)
 {
 	int i;
 
@@ -902,11 +968,11 @@ static int init_ring(struct nic *nic)
 	for (i = 0; i < TX_RING; i++)
 		tx_ring[i].FlagLen = 0;
 
-	np->cur_rx = 0;
+	np->cur_rx = RX_RING;
 	np->refill_rx = 0;
 	for (i = 0; i < RX_RING; i++)
 		rx_ring[i].FlagLen = 0;
-	return alloc_rx(nic);
+	alloc_rx(nic);
 }
 
 static void set_multicast(struct nic *nic)
@@ -948,7 +1014,7 @@ RESET - Reset the NIC to prepare for use
 static int forcedeth_reset(struct nic *nic)
 {
 	u8 *base = (u8 *) BASE;
-	int ret, oom, i;
+	int ret, i;
 	ret = 0;
 	dprintf(("forcedeth: open\n"));
 
@@ -966,7 +1032,7 @@ static int forcedeth_reset(struct nic *nic)
 	writel(0, base + NvRegAdapterControl);
 
 	/* 2) initialize descriptor rings */
-	oom = init_ring(nic);
+	init_ring(nic);
 
 	writel(0, base + NvRegLinkSpeed);
 	writel(0, base + NvRegUnknownTransmitterReg);
@@ -1053,7 +1119,7 @@ static int forcedeth_reset(struct nic *nic)
 		       base + NvRegPowerState);
 
 	pci_push(base);
-	udelay(10);
+	nv_udelay(10);
 	writel(readl(base + NvRegPowerState) | NVREG_POWERSTATE_VALID,
 	       base + NvRegPowerState);
 
@@ -1093,7 +1159,7 @@ static int forcedeth_reset(struct nic *nic)
 		printf("no link during initialization.\n");
 	}
 
-      out_drain:
+//      out_drain:
 	return ret;
 }
 
@@ -1112,7 +1178,9 @@ static int forcedeth_poll(struct nic *nic, int retrieve)
 	int len;
 	int i;
 	u32 Flags;
+	u32 valid;
 
+top:
 	i = np->cur_rx % RX_RING;
 
 	Flags = le32_to_cpu(rx_ring[i].FlagLen);
@@ -1129,6 +1197,7 @@ static int forcedeth_poll(struct nic *nic, int retrieve)
 			return 0;
 	}
 
+	valid = 1;
 	if (!retrieve)
 		return 1;
 
@@ -1140,6 +1209,8 @@ static int forcedeth_poll(struct nic *nic, int retrieve)
 */
 	wmb();
 	np->cur_rx++;
+	if (!valid)
+		goto top;
 	alloc_rx(nic);
 	return 1;
 }
@@ -1203,6 +1274,7 @@ static void forcedeth_disable(struct dev *dev __unused)
 	np->in_shutdown = 1;
 	stop_tx();
 	stop_rx();
+	txrx_reset(NULL);
 
 	/* disable interrupts on the nic or we will lock up */
 	writel(0, base + NvRegIrqMask);
@@ -1245,6 +1317,9 @@ static int forcedeth_probe(struct dev *dev, struct pci_device *pci)
 	int sz;
 	u8 *base;
 	int i;
+	u8 revision_id;
+	u32 powerstate;
+	int needs_mac_reset = 0;
 
 	if (pci->ioaddr == 0)
 		return 0;
@@ -1366,30 +1441,66 @@ static int forcedeth_probe(struct dev *dev, struct pci_device *pci)
 		else
 			np->tx_flags |= NV_TX2_LASTPACKET1;
 		break;
+	case 0x0268:
+		/* Fall Through */
+	case 0x0269:
+		/* Fall Through */
+	case 0x0372:
+		/* Fall Through */
+	case 0x0373:
+		//np->register_size = NV_PCI_REGSZ_VER2;
+
+		pci_read_config_byte(pci, PCI_REVISION_ID, &revision_id);
+
+		/* take phy and nic out of low power mode */
+		powerstate = readl(base + NvRegPowerState2);
+		powerstate &= ~NVREG_POWERSTATE2_POWERUP_MASK;
+		if ((pci->dev_id == PCI_DEVICE_ID_NVIDIA_NVENET_12 ||
+		     pci->dev_id == PCI_DEVICE_ID_NVIDIA_NVENET_13) &&
+		    revision_id >= 0xA3)
+			powerstate |= NVREG_POWERSTATE2_POWERUP_REV_A3;
+		writel(powerstate, base + NvRegPowerState2);
+		
+		//DEV_NEED_LASTPACKET1|DEV_IRQMASK_2|DEV_NEED_TIMERIRQ
+		np->irqmask = NVREG_IRQMASK_WANTED_2;
+		np->irqmask |= NVREG_IRQ_TIMER;
+//              np->need_linktimer = 1;
+//              np->link_timeout = jiffies + LINK_TIMEOUT;
+		if (np->desc_ver == DESC_VER_1)
+			np->tx_flags |= NV_TX_LASTPACKET1;
+		else
+			np->tx_flags |= NV_TX2_LASTPACKET1;
+
+		needs_mac_reset = 1;
+
+		break;
+
 	default:
 		printf
 		    ("Your card was undefined in this driver.  Review driver_data in Linux driver and send a patch\n");
 	}
 
 	/* find a suitable phy */
-	for (i = 1; i < 32; i++) {
+	for (i = 1; i <= 32; i++) {
 		int id1, id2;
-		id1 = mii_rw(nic, i, MII_PHYSID1, MII_READ);
+		int phyaddr = i & 0x1f;
+
+		id1 = mii_rw(nic, phyaddr, MII_PHYSID1, MII_READ);
 		if (id1 < 0 || id1 == 0xffff)
 			continue;
-		id2 = mii_rw(nic, i, MII_PHYSID2, MII_READ);
+		id2 = mii_rw(nic, phyaddr, MII_PHYSID2, MII_READ);
 		if (id2 < 0 || id2 == 0xffff)
 			continue;
 		id1 = (id1 & PHYID1_OUI_MASK) << PHYID1_OUI_SHFT;
 		id2 = (id2 & PHYID2_OUI_MASK) >> PHYID2_OUI_SHFT;
 		dprintf
 		    (("%s: open: Found PHY %hX:%hX at address %d.\n",
-		     pci->name, id1, id2, i));
-		np->phyaddr = i;
+		     pci->name, id1, id2, phyaddr));
+		np->phyaddr = phyaddr;
 		np->phy_oui = id1 | id2;
 		break;
 	}
-	if (i == 32) {
+	if (i == 33) {
 		/* PHY in isolate mode? No phy attached and user wants to
 		 * test loopback? Very odd, but can be correct.
 		 */
@@ -1397,12 +1508,13 @@ static int forcedeth_probe(struct dev *dev, struct pci_device *pci)
 		    ("%s: open: Could not find a valid PHY.\n", pci->name);
 	}
 
-	if (i != 32) {
+	if (i != 33) {
 		/* reset it */
 		phy_init(nic);
 	}
 	dprintf(("%s: forcedeth.c: subsystem: %hX:%hX bound to %s\n",
 		 pci->name, pci->vendor, pci->dev_id, pci->name));
+	if(needs_mac_reset) mac_reset(nic);
 	if(!forcedeth_reset(nic)) return 0; // no valid link
 //      if (board_found && valid_link)
 	/* point to NIC specific routines */
@@ -1427,6 +1539,10 @@ static struct pci_id forcedeth_nics[] = {
 	PCI_ROM(0x10de, 0x0057, "nforce9", "nForce NVENET_9 Ethernet Controller"),
 	PCI_ROM(0x10de, 0x0037, "nforce10", "nForce NVENET_10 Ethernet Controller"),
 	PCI_ROM(0x10de, 0x0038, "nforce11", "nForce NVENET_11 Ethernet Controller"),
+	PCI_ROM(0x10de, 0x0268, "nforce12", "nForce NVENET_12 Ethernet Controller"),
+	PCI_ROM(0x10de, 0x0269, "nforce13", "nForce NVENET_13 Ethernet Controller"),
+	PCI_ROM(0x10de, 0x0372, "nforce14", "nForce NVENET_14 Ethernet Controller"),
+	PCI_ROM(0x10de, 0x0373, "nforce15", "nForce NVENET_15 Ethernet Controller"),
 };
 static struct pci_driver forcedeth_driver __pci_driver = {
 	.type = NIC_DRIVER,.name = "forcedeth",.probe =
