@@ -639,7 +639,7 @@ natsemi_transmit(struct nic  *nic,
 
     to = currticks() + TX_TIMEOUT;
 
-    while (((tx_status=txd.cmdsts) & OWN) && (currticks() < to))
+    while ((((volatile u32) (tx_status=txd.cmdsts)) & OWN) && (currticks() < to))
         /* wait */ ;
 
     if (currticks() >= to) {
