@@ -245,6 +245,8 @@ print "EB_PXES\t:=\n";
 print "EB_ZPXES\t:=\n";
 print "EB_DSKS\t:=\n";
 print "EB_ZDSKS\t:=\n";
+print "EB_HDS\t:=\n";
+print "EB_ZHDS\t:=\n";
 print "EB_ELFS\t:=\n";
 print "EB_ZELFS\t:=\n";
 print "EB_LMELFS\t:=\n";
@@ -264,6 +266,7 @@ foreach my $pci (sort keys %pcient) {
 	print "EB_LILOS\t+= \$(BIN)/$img.lilo \nEB_ZLILOS\t+= \$(BIN)/$img.zlilo\n";
 	print "EB_PXES\t+= \$(BIN)/$img.pxe   \nEB_ZPXES\t+= \$(BIN)/$img.zpxe\n";
 	print "EB_DSKS\t+= \$(BIN)/$img.dsk   \nEB_ZDSKS\t+= \$(BIN)/$img.zdsk\n";
+	print "EB_HDS\t+= \$(BIN)/$img.hd   \nEB_ZHDS\t+= \$(BIN)/$img.zhd\n";
 	print "EB_ELFS\t+= \$(BIN)/$img.elf   \nEB_ZELFS\t+= \$(BIN)/$img.zelf\n";
 	print "EB_LMELFS\t+= \$(BIN)/$img.lmelf \nEB_ZLMELFS\t+= \$(BIN)/$img.zlmelf\n";
 	print "EB_ELFDS\t+= \$(BIN)/$img.elfd   \nEB_ZELFDS\t+= \$(BIN)/$img.zelfd\n";
@@ -283,6 +286,7 @@ foreach my $img (sort keys %buildent) {
 	print "EB_LILOS\t+= \$(BIN)/$img.lilo \nEB_ZLILOS\t+= \$(BIN)/$img.zlilo\n";
 	print "EB_PXES\t+= \$(BIN)/$img.pxe   \nEB_ZPXES\t+= \$(BIN)/$img.zpxe\n";
 	print "EB_DSKS\t+= \$(BIN)/$img.dsk   \nEB_ZDSKS\t+= \$(BIN)/$img.zdsk\n";
+	print "EB_HDS\t+= \$(BIN)/$img.hd   \nEB_ZHDS\t+= \$(BIN)/$img.zhd\n";
 	print "EB_ELFS\t+= \$(BIN)/$img.elf   \nEB_ZELFS\t+= \$(BIN)/$img.zelf\n";
 	print "EB_LMELFS\t+= \$(BIN)/$img.lmelf \nEB_ZLMELFS\t+= \$(BIN)/$img.zlmelf\n";
 	print "EB_ELFDS\t+= \$(BIN)/$img.elfd   \nEB_ZELFDS\t+= \$(BIN)/$img.zelfd\n";
@@ -321,7 +325,7 @@ foreach my $pci (sort keys %drivers) {
 	\$(CC) \$(CFLAGS) \$(\U$macro\EFLAGS) -o \$@ -c \$<
 
 \$(BIN)/$obj--%.o:	\$(BIN)/%.o \$(BIN)/$obj.o \$(MAKEDEPS)
-	\$(LD) -r \$(BIN)/$obj.o \$< -o \$@
+	\$(LD) \$(LDFLAGS) -r \$(BIN)/$obj.o \$< -o \$@
 
 EOF
 }
@@ -336,7 +340,7 @@ foreach my $isa (sort keys %isalist) {
 	\$(CC) \$(CFLAGS) \$(\U$macro\EFLAGS) -o \$@ -c \$<
 
 \$(BIN)/$isa--%.o:	\$(BIN)/%.o \$(BIN)/$isa.o \$(MAKEDEPS)
-	\$(LD) -r \$(BIN)/$isa.o \$< -o \$@ 
+	\$(LD) \$(LDFLAGS) -r \$(BIN)/$isa.o \$< -o \$@ 
 EOF
 }
 
